import 'package:hive/hive.dart';
import 'package:osam/src/domain/persist.dart';
import 'package:path_provider/path_provider.dart';

class PersistRepository<ST, US> implements Persist<ST, US> {

  Box<ST> _persist;
  Box<US> _uiPersist;
  Box<int> _versionBox;

  final String appName;
  final int versionNumber;

  PersistRepository({this.appName, this.versionNumber});

  @override
  Future<void> init() async {
    Hive.init((await getApplicationDocumentsDirectory()).path);
  }

  @override
  Future<void> initPersist() async {
    _persist = await Hive.openBox(appName);
    _uiPersist = await Hive.openBox(appName + '_ui_');
    _versionBox ??= await Hive.openBox(appName + '_version_');
  }

  @override
  Future<void> save({ST state, US uiState, int version}) async {
    if (state != null) await _persist.put('appState', state);
    if (uiState != null) await _uiPersist.put('uiState', uiState);
    if (version != null) await _versionBox.put('version', version);
  }

  @override
  ST get getState => _persist.get('appState');

  @override
  US get getUiState => _uiPersist.get('uiState');

  @override
  Future<int> get version async  {
    _versionBox ??= await Hive.openBox(appName + '_version_');
    return _versionBox.get('version');
  }

  @override
  Future<void> deleteHive() async => await Hive.deleteFromDisk();

  @override
  bool get isPersistEnabled => _persist == null ? false : true;
  
}
